// --------------------------------------------------------------------------------------------------------------------
// <copyright file="mdsautokey.h" company="Microsoft">
//  Copyright (c) Microsoft Corporation.  All rights reserved. 
// </copyright>
// <summary>
//   Header file for retrieving MDS keys using the autokey protocol
// </summary>
// --------------------------------------------------------------------------------------------------------------------
#ifndef _AUTOKEY_H_
#define _AUTOKEY_H_
#include <iostream>
#include <map>
#include <memory>
#include <vector>
#include <libxml2/libxml/tree.h>
#include <decryptor.hh>
    
    // Indicates the result status of requesting/parsing the latest MDS keys
namespace mdsautokey {
    enum autokeyResultStatus {
        autokeySuccess,
        autokeyPartialSuccess,
        autokeyFailure
    };

    class autokeyResult
    {
    private:
        std::string detail;
    public:
        autokeyResultStatus status;
        bool Success() const;
        bool Failure() const;
        std::string Details() const;
        // Constructors
        autokeyResult(autokeyResultStatus, std::string);
        autokeyResult();
        // Copy Constructor and Operator
        autokeyResult(const autokeyResult&);
        autokeyResult& operator=(const autokeyResult&);
        // Move Constructor and Operator
        autokeyResult(autokeyResult&&);
        autokeyResult& operator= (autokeyResult&&);
        // Operator for ostream
        friend std::ostream& operator<<(std::ostream&, const autokeyResult&);
    };

    using uniqueXmlDoc = std::unique_ptr<xmlDoc, void(*)(xmlDocPtr)>;
    using keyAndDecryptor = std::pair<std::string, cryptutil::decryptor>;

    // Exception occurs if invalid XML is parsed
    class xmlParseException : public std::exception
    {
        std::string exMessage;
    public:
        xmlParseException(std::string& errDetail)
            : exMessage(std::string("Error occurred while parsing file: ") + errDetail)
        {}
        virtual const char* what() const throw()
        {
            return exMessage.c_str();
        }
    };

    // Exception occurs if file is unreadable
    class fileException : public std::exception
    {
        std::string exMessage;
    public:
        fileException(std::string& errDetail)
            : exMessage(std::string("Error occurred while reading file: ") + errDetail)
        {}
        virtual const char* what() const throw()
        {
            return exMessage.c_str();
        }
    };

    // Default autokey helper config file
    const std::string autokeyDefaultCfg = "/etc/mdsd.d/mdsautokey.cfg";

    // Functions
    void DownloadFile(const std::string&, const std::string&);
    uniqueXmlDoc ReadXmlFile(const std::string&);
    std::string ParseCommandBlob(xmlNode*, const std::string&, const cryptutil::decryptor&);
    void ParseKeyListXml(xmlNode*, std::string&, std::map<std::pair<std::string, std::string>, std::string>&, const cryptutil::decryptor&);
    autokeyResult GetLatestMdsKeys(const std::string&, const std::string&, int, std::map<std::pair<std::string, std::string>, std::string>&);
    autokeyResult ParseConfigFile(const std::string&, std::vector<keyAndDecryptor>&);
    autokeyResult ValidateConfig(const std::string&, const std::string&, int, std::vector<keyAndDecryptor>&);
    int CountSubstringOccurrences(const std::string&, const std::string&);
    std::string EscapeAmpersands(const std::string&);
}
#endif